package com.sunil.aws_helper

import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import com.amazonaws.auth.BasicAWSCredentials
import com.amazonaws.mobileconnectors.s3.transferutility.TransferListener
import com.amazonaws.mobileconnectors.s3.transferutility.TransferObserver
import com.amazonaws.mobileconnectors.s3.transferutility.TransferState
import com.amazonaws.mobileconnectors.s3.transferutility.TransferUtility
import com.amazonaws.regions.Region
import com.amazonaws.regions.Regions
import com.amazonaws.services.s3.AmazonS3Client
import com.amazonaws.services.s3.model.CannedAccessControlList
import com.amazonaws.services.s3.model.ObjectMetadata

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.io.File

/** AwsHelperPlugin */
class AwsHelperPlugin : FlutterPlugin, MethodCallHandler, EventChannel.StreamHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private val TAG = "AwsHelperPlugin"
    private val CHANNEL = "com.sunil.aws_helper/aws_helper"
    private val STREAM = "com.sunil.aws_helper/uploading_status"
    private var awsKey: String? = null
    private var awsSecret: String? = null
    private var region: String? = null
    private var file_path: String? = null
    private var bucketPath: String? = null
    private var parentResult: Result? = null
    private var mContext: Context? = null
    private var eventChannel: EventChannel? = null
    private var methodChannel: MethodChannel? = null
    private var events: EventChannel.EventSink? = null
    private var transferUtility: TransferUtility? = null
    private var transferObserver: TransferObserver? = null
    private var fileException: java.lang.Exception? = null
    private var bucket_url: String? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        this.mContext = flutterPluginBinding.applicationContext
        methodChannel = MethodChannel(flutterPluginBinding.binaryMessenger, CHANNEL)
        eventChannel = EventChannel(flutterPluginBinding.binaryMessenger, STREAM)
        eventChannel!!.setStreamHandler(this)
        methodChannel!!.setMethodCallHandler(this)

        Log.d(TAG, "whenAttachedToEngine")
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel!!.setMethodCallHandler(null)
        methodChannel = null
        eventChannel!!.setStreamHandler(null)
        eventChannel = null

        Log.d(TAG, "onDetachedFromEngine")
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "uploadImageToS3") {
            parentResult = result
            awsKey = call.argument<String>("awsKey")
            awsSecret = call.argument<String>("awsSecret")
            region = call.argument<String>("region")
            file_path = call.argument<String>("file_path")
            bucketPath = call.argument<String>("bucketPath")

            try {
                val regionName: String = region!!.replaceFirst("Regions.".toRegex(), "")
                val region = Regions.valueOf(regionName)
                val credentials = BasicAWSCredentials(awsKey, awsSecret)
                val s3Client = AmazonS3Client(credentials)
//                s3Client.endpoint = "https://s3-${Region.getRegion(region)}.amazonaws.com/";
                Log.d(TAG, "onMethodCall: ${Region.getRegion(region)}")
                s3Client.setRegion(Region.getRegion(region))
                transferUtility = TransferUtility.builder().s3Client(s3Client).context(mContext).build()
                uploadImage()
            } catch (e: Exception) {
                Log.d(TAG, "onMethodCall: " + e.message)
            }
        } else {
            result.notImplemented()
        }
    }

    private fun uploadImage() {
        if (isValidFile(file_path)) {
            val file: File = File(file_path)
            transferObserver = transferUtility!!.upload(
                    bucketPath,
                    file.name,
                    file,
                    ObjectMetadata(),
                    CannedAccessControlList.PublicRead, object : TransferListener {
                override fun onStateChanged(id: Int, state: TransferState) {
                    if (transferObserver != null && transferObserver!!.id == id) {
                        Log.d(TAG, "file onStateChanged: " + transferObserver!!.state)
                        if (TransferState.COMPLETED == transferObserver!!.state) {
                            transferObserver!!.setTransferListener(null)
                            bucket_url = transferObserver!!.bucket + "/" +
                                    transferObserver!!.key
                            Log.d(TAG, "file url: $bucket_url")
                        } else if (TransferState.FAILED == transferObserver!!.state || TransferState.CANCELED == transferObserver!!.state || TransferState.PENDING_CANCEL == transferObserver!!.state || TransferState.UNKNOWN == transferObserver!!.state) {
                            transferObserver!!.setTransferListener(null)
                            fileException = RuntimeException("File upload fail. State: " +
                                    transferObserver!!.state.name)
                        }
                        if (transferObserver != null) {
                            if (fileException != null || transferObserver!!.state == TransferState.COMPLETED) {
                                Log.d(TAG, "file uploadComplete from file onStateChanged")
                                parentResult!!.success(bucket_url)
                            }
                        }
                        return
                    }
                }

                override fun onProgressChanged(id: Int, bytesCurrent: Long, bytesTotal: Long) {
                    val percentDoNef = bytesCurrent.toFloat() / bytesTotal.toFloat() * 100
                    val percentDone = percentDoNef.toInt()
                    Log.d(TAG, "ID: bytesTotal: $bytesTotal   $percentDone%")

                    if (events != null) {
                        events!!.success(percentDone)
                    }
                }

                override fun onError(id: Int, ex: java.lang.Exception?) {
                    if (transferObserver != null && transferObserver!!.id == id) {
                        fileException = ex
                        Log.d(TAG, "file uploading onError: " + ex!!.message)
                    }

                    if (fileException != null) {
                        if (transferObserver != null) {
                            transferObserver!!.setTransferListener(null)
                        }
                        Log.d(TAG, "file uploadComplete onError: " + ex!!.message)
                        parentResult!!.error("-1", ex!!.message, fileException)
                    }
                    invalidateEventSink()
                }
            })
        }
    }

    fun isValidFile(file_path: String?): Boolean {
        if (file_path.isNullOrEmpty()) {
            Log.d(TAG, "isValidFile: file_path, bucket_path should not be null or blank")
            return false
        }
        if (!file_path.isNullOrEmpty()) {
            val file: File = File(file_path)
            if (!file.exists()) {
                Log.d(TAG, "thumb_file_path not exists")
                return false
            }
        }
        return true
    }

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        this.events = events!!
    }

    override fun onCancel(arguments: Any?) {
        invalidateEventSink()
    }

    private fun invalidateEventSink() {
        if (events != null) {
            events!!.endOfStream()
            events = null
        }
    }
}
