
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class AwsHelper {

  final String awsKey;
  final String awsSecret;
  final Regions region;
  final String filePath;
  final String bucketPath;

  AwsHelper({
    @required this.awsKey,
    @required this.awsSecret,
    this.region = Regions.US_WEST_2,
    @required this.filePath,
    @required this.bucketPath,
  });

  static const MethodChannel _channel =
  const MethodChannel('com.sunil.aws_helper/aws_helper');

  static const EventChannel _eventChannel =
  const EventChannel('com.sunil.aws_helper/uploading_status');

  Future<String> get uploadFile async {
    Map<String, dynamic> args = <String, dynamic>{};
    args.putIfAbsent("awsKey", () => awsKey);
    args.putIfAbsent("awsSecret", () => awsSecret);
    args.putIfAbsent("region", () => region.toString());
    args.putIfAbsent("file_path", () => filePath);
    args.putIfAbsent("bucketPath", () => bucketPath);

    debugPrint("AwsS3Plugin: file path is: $filePath");

    final String result = await _channel.invokeMethod('uploadImageToS3', args);

    return result;
  }

  Stream get getUploadStatus => _eventChannel.receiveBroadcastStream();

}

/// Enumeration of region names
enum Regions {
  GovCloud,
  US_GOV_EAST_1,
  US_EAST_1,
  US_EAST_2,
  US_WEST_1,
  US_WEST_2, ///Default: The default region of AWS Android SDK
  EU_WEST_1,
  EU_WEST_2,
  EU_WEST_3,
  EU_CENTRAL_1,
  EU_NORTH_1,
  AP_EAST_1,
  AP_SOUTH_1,
  AP_SOUTHEAST_1,
  AP_SOUTHEAST_2,
  AP_NORTHEAST_1,
  AP_NORTHEAST_2,
  SA_EAST_1,
  CA_CENTRAL_1,
  CN_NORTH_1,
  CN_NORTHWEST_1,
  ME_SOUTH_1
}